@@ -479,7 +479,7 @@ class StorageEvent(HookEvent):
 
         if isinstance(self.storage, model.Storage):
             snapshot["storage_name"] = self.storage.name
-            snapshot["storage_id"] = self.storage.id
+            snapshot["storage_index"] = self.storage.index
             snapshot["storage_location"] = str(self.storage.location)
         return snapshot
 
@@ -489,13 +489,16 @@ class StorageEvent(HookEvent):
         Not meant to be called by charm code.
         """
         storage_name = snapshot.get("storage_name")
-        storage_id = snapshot.get("storage_id")
+        storage_index = snapshot.get("storage_index")
         storage_location = snapshot.get("storage_location")
 
-        if storage_name and storage_id is not None:
-            self.storage = next(
-                (s for s in self.framework.model.storages[storage_name] if s.id == storage_id),
-                None,)
+        if storage_name and storage_index is not None:
+            storages = self.framework.model.storages[storage_name]
+            self.storage = next((s for s in storages if s.index == storage_index), None,)
+            if self.storage is None:
+                msg = 'failed loading storage (name={!r}, index={!r}) from snapshot' \
+                    .format(storage_name, storage_index)
+                raise RuntimeError(msg)
             self.storage.location = storage_location
 
 
@@ -773,7 +776,10 @@ class CharmMeta:
 
     """
 
-    def __init__(self, raw: dict = {}, actions_raw: dict = {}):
+    def __init__(self, raw: dict = None, actions_raw: dict = None):
+        raw = raw or {}
+        actions_raw = actions_raw or {}
+
         self.name = raw.get('name', '')
         self.summary = raw.get('summary', '')
         self.description = raw.get('description', '')
