@@ -84,13 +84,15 @@ class SymlinkTargetError(Exception):
 class EventSpec:
     def __init__(self, event_type, event_name, env_var=None,
                  relation_id=None, remote_app=None, remote_unit=None,
-                 model_name=None, set_in_env=None, workload_name=None):
+                 model_name=None, set_in_env=None, workload_name=None,
+                 departing_unit_name=None):
         self.event_type = event_type
         self.event_name = event_name
         self.env_var = env_var
         self.relation_id = relation_id
         self.remote_app = remote_app
         self.remote_unit = remote_unit
+        self.departing_unit_name = departing_unit_name
         self.model_name = model_name
         self.set_in_env = set_in_env
         self.workload_name = workload_name
@@ -382,8 +384,12 @@ class _TestMain(abc.ABC):
             remote_unit = event_spec.remote_unit
             if remote_unit is None:
                 remote_unit = ''
-
             env['JUJU_REMOTE_UNIT'] = remote_unit
+
+            departing_unit_name = event_spec.departing_unit_name
+            if departing_unit_name is None:
+                departing_unit_name = ''
+            env['JUJU_DEPARTING_UNIT'] = departing_unit_name
         else:
             env.update({
                 'JUJU_REMOTE_UNIT': '',
@@ -462,7 +468,8 @@ class _TestMain(abc.ABC):
         ), (
             EventSpec(RelationJoinedEvent, 'db_relation_joined',
                       relation_id=1,
-                      remote_app='remote', remote_unit='remote/0'),
+                      remote_app='remote',
+                      remote_unit='remote/0'),
             {'relation_name': 'db',
              'relation_id': 1,
              'app_name': 'remote',
@@ -470,7 +477,8 @@ class _TestMain(abc.ABC):
         ), (
             EventSpec(RelationChangedEvent, 'mon_relation_changed',
                       relation_id=2,
-                      remote_app='remote', remote_unit='remote/0'),
+                      remote_app='remote',
+                      remote_unit='remote/0'),
             {'relation_name': 'mon',
              'relation_id': 2,
              'app_name': 'remote',
@@ -478,7 +486,8 @@ class _TestMain(abc.ABC):
         ), (
             EventSpec(RelationChangedEvent, 'mon_relation_changed',
                       relation_id=2,
-                      remote_app='remote', remote_unit=None),
+                      remote_app='remote',
+                      remote_unit=None),
             {'relation_name': 'mon',
              'relation_id': 2,
              'app_name': 'remote',
@@ -486,11 +495,14 @@ class _TestMain(abc.ABC):
         ), (
             EventSpec(RelationDepartedEvent, 'mon_relation_departed',
                       relation_id=2,
-                      remote_app='remote', remote_unit='remote/0'),
+                      remote_app='remote',
+                      remote_unit='remote/0',
+                      departing_unit_name='remote/42'),
             {'relation_name': 'mon',
              'relation_id': 2,
              'app_name': 'remote',
-             'unit_name': 'remote/0'},
+             'unit_name': 'remote/0',
+             'departing_unit_name': 'remote/42'},
         ), (
             EventSpec(RelationBrokenEvent, 'ha_relation_broken',
                       relation_id=3),
@@ -516,11 +528,13 @@ class _TestMain(abc.ABC):
         ), (
             EventSpec(RelationDepartedEvent, 'mon_relation_departed',
                       relation_id=2,
-                      remote_unit='remote/0'),
+                      remote_unit='remote/0',
+                      departing_unit_name='remote/42'),
             {'relation_name': 'mon',
              'relation_id': 2,
              'app_name': 'remote',
-             'unit_name': 'remote/0'},
+             'unit_name': 'remote/0',
+             'departing_unit_name': 'remote/42'},
         ), (
             EventSpec(ActionEvent, 'start_action',
                       env_var='JUJU_ACTION_NAME'),
